(* E37UM* For noForth C&V FR59xx lp.0, USCI I2C on MSP430FR59xx using pull-ups.
   The bitrate values are for an 8 MHz DCO, for 16 MHz they should be doubled.
   This is a better version, the routines work more solid. Code Vsn 1.02

  The most hard to find data are those for the selection registers.
  To find the data for the selection register of Port-1 here 026 and
  041 you have to go to the "Port Schematics". For P1.6 and P1.7 these
  are page 48 and 49 of SLAS735J.PDF These tables say which function
  will be on each I/O-bit at a specific setting of the registers.

  Address 20A - P1SEL0, port-1 selection register-0 = 00
  Address 20C - P1SEL1, port-1 selection register-1 = C0

  A description of USCI as I2C can be found in SLAU144J.PDF from page
  449 to page 473, the register description starts at page 468. The
  UCB0CTL0 register selects the SPI- or I2C-mode we are in.

 Connect the I2C-print from the Forth users group or any other module
  with I2C chips. Connect the power lines, P1.7 to SDA and P1.6 to SCL,
  note that two 10k pullup resistors has te be mounted and jumper P1.6 to
  the green led has to be removed, that's it.
  User words: {I2WRITE  {I2READ)  {I2READ  I2STOP}  I2IN
              I2IN}  I2OUT1  I2OUT  I2C?  SETUP-I2C

        **** New version with less primitives ****
        **** And assembly changed to hex code ****

  An example, first execute SETUP-I2C  After that the I2C is setup as a
  master. Sent byte 'b' to an I2C device with address 'a'.
    : >SLAVE    ( b a -- )  {i2write  i2out1  i2stop} ;
    : >PCF8574  ( b -- )    40 >slave ;

 Addresses, Lables and Bit patterns
 015C    - WDTCL        - Off allready
 020A    - P1SEL0       - 000
 020C    - P1SEL1       - 0C0
 0640    - UCB0CTLW0    - F91
 0642    - UCB0CTLW1    - 081
 0646    - UCB0BR0      - 0A0
 0647    - UCB0BR1      - 000
 0648    - UCB0STATW    - USCI status
 064C    - UCB0RXBUF    - RX Data
 064E    - UCB0TXBUF    - TX Data
 0654    - UCB0I2C0A    - NC
 0660    - UCB0I2CSA    - 042
 066A    - UCB0IE       - USCI interrupt enable
 066C    - UCB0IFG      - 002 = TX ready, 001 = RX ready
 *)

hex  inside also
\ code INT-OFF  C232 ,  4F00 ,  end-code

: SETUP-I2C ( -- )
    int-off
    C0 20A *bic     \ P1SEL0    I2C to pins
    C0 20C *bis     \ P1SEL1
    C0 206 *bic     \ P1REN
    C0 202 *bic     \ P1OUT
    0001 640 **bis  \ UCB0CTLW0 reset USCI
    0FD1 640 !      \ UCB0CTLW0 I2C master
    dm 160 646 !    \ UCB0BRW   Bitrate 100 KHz with 16 MHz DCO
    01 640 **bic    \ UCB0CTL0  Resume USCI
    02 66C **bis ;  \ UCB0IFG   Start with TX buffer empty

: I2C   ( fl -- )   ?abort ;        \ I2C error message

\ 01 = wait for data received to RX; 02 = wait for TX to be sent
\ : I2READY   ( bit -- )
\    80  begin
\        over 66C bit** if  2drop exit  then
\    1- dup 0= until  true ?abort ;
code I2READY   ( bit -- )
    FFF # day  mov      \ 512 to counter (day)
    begin,
        tos 66C & bit  cs? if,  sp )+ tos mov  next  then,
        #1 day sub
    0=? until,
    chere >r  \ Reuse of code
    ip push
    ' i2c >body # ip mov
    next
end-code

\ 02 = wait for startcond. to finish; 04 = wait for stopcond. to finish
\ : I2DONE    ( bit -- )  \ wait until startcond. or stopcond. is done
\    80  begin
\        over 640 bit** 0= if  2drop exit  then
\    1- dup 0= until  true ?abort ;
code I2DONE     ( bit -- )
    FFF # day mov
    begin,
        tos 640 & bit  cc? if,  sp )+ tos mov  next  then,
        #1 day sub
    0=? until,
    r> jmp
end-code

\ : I2START   ( -- )      02 640 **bis ;        \ UCB0CTLW0
\ : I2STOP    ( -- )      04 640 **bis ;        \ UCB0CTLW0
\ : I2NACK    ( -- )      08 640 **bis ;        \ UCB0CTLW0
\ : I2ACK?    ( -- fl )   20 66C bit** 0= ;     \ UCB0STAT
: I2OUT1    ( b -- )    64E !  02 i2done  noop ;  \ UCB0TXBUF Send first data to TX
: I2OUT     ( b -- )    64E !  02 i2ready  noop ; \ UCB0TXBUF TX to shiftreg.
: I2IN      ( -- b )    01 i2ready  64C @ ;     \ UCB0RXBUF Read databyte
: I2IN}     ( -- b )    04 640 **bis  i2in  04 i2done ;  \ UCB0CTLW0 Read last I2C databyte
: I2STOP}   ( -- )      04 640 **bis 3 . 04 i2done 4 . ; \ UCB0CTLW0 Stop condition & check
: >DEV      ( a -- )    2/ 660 ! ;              \ UCB0I2CSA Set I2C device address

: {I2WRITE) ( -- )              \ Send I2C device address for writing
    12 640 **bis  02 i2ready ;  \ UCB0CTLW0  Send start condition

: {I2READ)  ( -- )              \ Send I2C device address for reading
    10 640 **bic  02 640 **bis  \ UCB0CTLW0  Setup read & start
    02 i2ready  02 i2done ;     \ Wait for start condition & ack

: {I2WRITE  ( a -- )            \ Send I2C device address for writing
    >dev  {i2write) ;           \ UCB0CTLW0  Send start condition

: {I2READ   ( a -- )            \ Set and send I2C device address for reading
    >dev   {i2read) ;           \ UCB0I2CSA Set slave address


: TEST-ACK? ( a -- f )           \ Test if device is accessable
    {i2write  i2stop}  20 66C bit** 0= ;

\ Prints -1 if device with address 'a' is present on I2C-bus otherwise 0.
: I2?      ( a -- )            \ Result is true when device 'a' is on I2C bus
    setup-i2c  test-ack? . ;   \ Address device, present?

setup-i2c  fresh
shield HWI2C\   freeze

\ End
