(* Simple binary BSL replacement loader for MSP430F149.
   Does password and erase using the default BSL, then a short relocatable 
   RAM BSL is sent to RAM using the default loader.
   The RAM BSL than is used to program the flash using data from EEPROM.

   Functional steps:
        - Reset to the TI BSL
        - Send password
        - Erase flash segments
        - Load own BSL programmer from address 200
        - Start own BSL programmer (jump to 200)
        - Send data using multiple ( addr len -- ) binary parts
        - Then start MPU again

MSP430 registers in use:
PC      - Subr. stack
IP      - Address
TOS     - Char
DAY     - Counter to read data word
MOON    - Scratch for BE and BK
SUN     - 16-bits data
XX      - Delay counter for BE and BK
YY      - Counter for Flash write routine
*) 

\ Software RS232 subroutine labels for BSL with addr. correction
value WBI  value BEM  value BKE  value OFFSET
: WB  wbi offset -  220 + ;
: BE  bem offset -  220 + ;
: BK  bke offset -  220 + ;

: *SETUP-BSL    ( -- )
    5A80 # 0120 & mov       \ WDTCTL   Watchdog off
    #0 0000 & .b mov        \ IE1      Erase interrupt flags
    220 # rp mov            \          Init. RP
\ Activate I/O ports, leds & switch
    FF # 002A & .b mov      \ P2DIR    All bits output
    3C # 0029 & .b mov      \ P2OUT    Set 4 leds on
\ Klok selectie en test
    80 # 0057 & .b bic      \ BCSCTL1  XT2 on
    begin,                  \ Wait for osc. to start.
        #2 0002 & .b bic    \ IFG1     Reset osc. error flag
        FF # sun mov        \ Start wait loop
        begin,
            #-1 sun add
        0=? until,
        #2 0002 & .b bit    \ IFG1     Stop when error flag is zero
    cc? until,
    88 # 0058 & .b bis      \ BCSCTL2  Mclk = SMclk = XT2
\ Software RS232
    #4 002A & .b bic        \ P2DIR    P2.2 is input
    #4 0029 & .b bis        \ P2OUT    High
    0F # 0022 & .b bis      \ P1DIR    P1.1 is output
    #2 0021 & .b bis        \ P1OUT    P1.1 starts high
\ flash initialisation
    A554 # 012A & mov ;     \ FCTL2    Mclk/xx for flash timing

\ Write one dataword from SUN to Flash memory at address in IP
: *ROM!         ( sun=x ip=addr -- )
\   begin,  #1 12C & bit  cc? until, \ FCTL3  Busy?
    A500 # 012C & mov           \ FCTL3
    A500 0040 + # 0128 & mov    \ FCTL1
    sun ip ) mov
    begin,  #1 12C & bit  cc? until, \ FCTL3  Busy?
    A500 # 0128 & mov           \ FCTL1
    A500 0010 + # 012C & mov    \ FCTL3
    #2 ip add  ;                \ Increase address

\ Use only with uppercase characters
: *KEY          ( -- tos=ch )       bk # call ;
: *EMIT         ( tos=ch -- )       be # call ;

\ Receive a 16-bit dataword using four uppercase ASCII chars
: *WORD         ( -- sun=16-bit word)
    #4 day mov
    begin,
        sun sun add  sun sun add
        sun sun add  sun sun add
        *key  ch 0 # tos sub
        0A # tos cmp
        <eq? if,  7 # tos sub  then,
        tos sun bis
        #1 day sub
    0=? until, ;

\ Write a binary (part) with a length of YY cells to Flash
\ Register IP contains the start address, YY is length in cells
\ First the length has to be send, after that length cells!
: *WRITE        ( -- )      \ Write data block to Flash
    *word  sun yy mov       \ Get & set data loop counter
    begin,
        *word  *rom!        \ Get & write data word
        #1 yy sub           \ Decrease data counter
    0=? until, ;

\ Simple BSL command interpreter, only 2 commands
\ 1 = "A"   Set work address
\ 2 = "W"   Write memory from address set by A
code COMMAND    ( -- )
chere to offset
    ahead,
chere to wbi    ( -- )          \ Wait bittime
      dm 268 # xx mov
      begin,  #1 xx sub  0=? until,
      rp )+ pc mov  ( ret )
chere to bem    ( char -- )     \ RS232s Char to RS232
      100 # tos bis             \ Bit-8 is stopbit
      tos tos add               \ Make room for startbit
      0A # moon mov             \ Start + 8 + Stop bit
      begin,
        tos rrc                 \ Get next bit to carry
        cs? if,
          #2 21 & .b bis        \ P1OUT  Send one
        else,
          #2 21 & .b bic        \ P1OUT  Send zero
        then,
        wb # call               \ Wait bittime
        #1 moon sub             \ Send all bits
      0=? until,
      rp )+ pc mov  ( ret )
chere to bke    ( -- char )     \ rs232s read upc & number char from rs232
      begin,
        #4 28 & .b bit          \ P2IN  wait for startbit
      cc? until,
      30 # moon mov             \ wait extra to reach sample point
      begin,  #1 moon sub  =? until,
      #8 moon mov               \ read 8 databits
      begin,                    \ Leave stopbit
        wb # call               \ wait bittime
        #4 28 & .b bit          \ P2IN  read rx line to carry
        tos rrc                 \ shift rx into char
        #1 moon sub             \ return at stopbit
      0=? until,
      wb # call                 \ wait bittime for stopbit
      tos swpb
      #-1 tos .b bia            \ Use low byte only
      rp )+ pc mov  ( ret )
    then,
    *setup-bsl                  \ Init. all
    #0 tos mov                  \ Show startup
    begin,
      0 dox x) 0 dox x) mov
      #1 tos sub
    0=? until,
    begin,                      \ Main loop
      #-1 0029 & .b bix         \ P2OUT    Invert led pattern
      *key                      \ Ask char
      ch A # tos cmp  =? if,
        *word  sun ip mov       \ Get & set address
      else,
        ch W # tos cmp  =? if,
          *write                \ Get & write data block
      then, then,
    again,
end-code

(* TI code conversion "Custom BSL-01"  length: 354 bytes

2A 3C
3B 40 0C 01 1B 83 FE 23
30 41 37 D0 00 01 07 57
39 40 0A 00 07 10 03 28
E2 D3 21 00 02 3C E2 C3
21 00 B0 12 22 02 19 83
F5 23 30 41 E2 B2 28 00
FD 2F 39 40 30 00 19 83
FE 23 39 42 B0 12 22 02
E2 B2 28 00 07 10 19 83
F9 23 B0 12 22 02 87 10
77 F3 30 41 B2 40 80 5A
20 01 C2 43 00 00 31 40
20 02 F2 40 FF 00 2A 00
F2 40 3C 00 29 00 F2 C0
80 00 57 00 E2 C3 02 00
3A 40 FF 00 3A 53 FE 23
E2 B3 02 00 F7 2F F2 D0
88 00 58 00 E2 C2 2A 00
E2 D2 29 00 F2 D0 0F 00
22 00 E2 D3 21 00 B2 40
54 A5 2A 01 07 43 9E 4E
00 00 00 00 17 83 FB 23
F2 E3 29 00 B0 12 4E 02
37 90 41 00 13 20 28 42
0A 5A 0A 5A 0A 5A 0A 5A
B0 12 4E 02 37 80 30 00
37 90 0A 00 02 38 37 80
07 00 0A D7 18 83 F0 23
05 4A 3D 3C 37 90 57 00
3A 20 28 42 0A 5A 0A 5A
0A 5A 0A 5A B0 12 4E 02
37 80 30 00 37 90 0A 00
02 38 37 80 07 00 0A D7
18 83 F0 23 0C 4A 28 42
0A 5A 0A 5A 0A 5A 0A 5A
B0 12 4E 02 37 80 30 00
37 90 0A 00 02 38 37 80
07 00 0A D7 18 83 F0 23
92 B3 2C 01 FD 2F B2 40
00 A5 2C 01 B2 40 40 A5
28 01 85 4A 00 00 92 B3
2C 01 FD 2F B2 40 00 A5
28 01 B2 40 10 A5 2C 01
25 53 1C 83 D8 23 A8 3F
q

*)
